import 'package:senior_platform_authentication/senior_platform_authentication.dart';

import '../datasources/secure_storage_local_datasource.dart';

abstract class SecureStorageRepository {
  Future<void> deleteUser();
  Future<void> deleteToken();
  Future<void> deleteTokenExpirationDate();
  Future<User?> readUser();
  Future<Token?> readToken();
  Future<DateTime?> readTokenExpirationDate();
  Future<void> writeUser(User user);
  Future<void> writeToken(Token user);
  Future<void> writeTokenExpirationDate(DateTime tokenExpirationDate);
}

class SecureStorageRepositoryImpl implements SecureStorageRepository {
  late final SecureStorageDatasource _datasource;

  SecureStorageRepositoryImpl({SecureStorageDatasource? datasource}) {
    _datasource = datasource ?? SecureStorageLocalDatasource();
  }

  @override
  Future<User?> readUser() async {
    final userModel = await _datasource.readUser();

    if (userModel != null) {
      return User(
        changePassword: userModel.changePassword,
        admin: userModel.admin,
        allowedToChangePassword: userModel.allowedToChangePassword,
        id: userModel.id,
        username: userModel.username,
        fullName: userModel.fullName,
        email: userModel.email,
        tenantDomain: userModel.tenantDomain,
        tenantName: userModel.tenantName,
        tenantLocale: userModel.tenantLocale,
        blocked: userModel.blocked,
        properties: userModel.properties,
      );
    }

    return null;
  }

  @override
  Future<Token?> readToken() async {
    final tokenModel = await _datasource.readToken();

    if (tokenModel != null) {
      return Token(
        version: tokenModel.version,
        expiresIn: tokenModel.expires_in,
        username: tokenModel.username,
        tokenType: tokenModel.token_type,
        accessToken: tokenModel.access_token,
        refreshToken: tokenModel.refresh_token,
        type: tokenModel.type,
        email: tokenModel.email,
        fullName: tokenModel.fullName,
        tenantName: tokenModel.tenantName,
        locale: tokenModel.locale,
      );
    }

    return null;
  }

  @override
  Future<DateTime?> readTokenExpirationDate() async {
    return await _datasource.readTokenExpirationDate();
  }

  @override
  Future<void> writeUser(User user) async {
    final userModel = UserModel(
      changePassword: user.changePassword,
      admin: user.admin,
      allowedToChangePassword: user.allowedToChangePassword,
      id: user.id,
      username: user.username,
      fullName: user.fullName,
      email: user.email,
      tenantDomain: user.tenantDomain,
      tenantName: user.tenantName,
      tenantLocale: user.tenantLocale,
      blocked: user.blocked,
      properties: user.properties,
    );

    await _datasource.writeUser(userModel);
  }

  @override
  Future<void> writeToken(Token token) async {
    final tokenModel = TokenModel(
      version: token.version,
      expires_in: token.expiresIn,
      username: token.username,
      token_type: token.tokenType,
      access_token: token.accessToken,
      refresh_token: token.refreshToken,
      type: token.type,
      email: token.email,
      fullName: token.fullName,
      tenantName: token.tenantName,
      locale: token.locale,
    );

    await _datasource.writeToken(tokenModel);
  }

  @override
  Future<void> writeTokenExpirationDate(DateTime tokenExpirationDate) async {
    await _datasource.writeTokenExpirationDate(tokenExpirationDate);
  }

  @override
  Future<void> deleteUser() async {
    await _datasource.deleteUser();
  }

  @override
  Future<void> deleteToken() async {
    await _datasource.deleteToken();
  }

  @override
  Future<void> deleteTokenExpirationDate() async {
    await _datasource.deleteTokenExpirationDate();
  }
}
