import 'dart:developer';

import 'package:senior_platform_authentication/senior_platform_authentication.dart';

import '../../data/repositories/secure_storage_repository.dart';
import '../entities/authentication_data.dart';
import 'clear_stored_data_usecase.dart';
import 'store_authentication_data_usecase.dart';

class RefreshStoredTokenUsecase implements BaseUsecase<Token?, NoParams> {
  late final SecureStorageRepository _secureStorageRepository;
  late final RefreshTokenUsecase _refreshTokenUsecase;
  late final GetUserUsecase _getUserUsecase;
  late final StoreAuthenticationDataUsecase _storeAuthenticationDataUsecase;
  late final ClearStoredDataUsecase _clearStoredDataUsecase;

  RefreshStoredTokenUsecase({
    SecureStorageRepository? secureStorageRepository,
    required RefreshTokenUsecase refreshTokenUsecase,
    required GetUserUsecase getUserUsecase,
    required StoreAuthenticationDataUsecase storeAuthenticationDataUsecase,
    required ClearStoredDataUsecase clearStoredDataUsecase,
  }) {
    _secureStorageRepository =
        secureStorageRepository ?? SecureStorageRepositoryImpl();
    _refreshTokenUsecase = refreshTokenUsecase;
    _getUserUsecase = getUserUsecase;
    _storeAuthenticationDataUsecase = storeAuthenticationDataUsecase;
    _clearStoredDataUsecase = clearStoredDataUsecase;
  }

  @override
  Future<Token?> call(NoParams params) async {
    try {
      var storedToken = await _secureStorageRepository.readToken();
      final storedUser = await _secureStorageRepository.readUser();
      final tenant = storedUser?.tenantName ?? '';

      // Invalid token
      if ((storedToken?.refreshToken.isEmpty ?? true) ||
          (storedToken?.accessToken.isEmpty ?? true)) {
        await _clearStoredDataUsecase(NoParams());
        return null;
      }

      final refreshToken = RefreshToken(
        token: storedToken!.refreshToken,
        scope: '',
        tenant: tenant,
      );

      final authenticationResponse = await _refreshTokenUsecase(refreshToken);

      bool stored = false;
      if (authenticationResponse.token != null) {
        final refreshedUser =
            await _getUserUsecase(authenticationResponse.token!.accessToken);

        stored = await _storeAuthenticationDataUsecase(
          AuthenticationData(
            user: refreshedUser,
            token: authenticationResponse.token,
          ),
        );
      }

      if (!stored) {
        await _clearStoredDataUsecase(NoParams());
        return null;
      }

      return authenticationResponse.token;
    } catch (e) {
      log('REFRESH TOKEN ERROR >>>> ${e.toString()}');
      await _clearStoredDataUsecase(NoParams());
      return null;
    }
  }
}
