import 'dart:async';

import 'package:equatable/equatable.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:senior_platform_authentication/senior_platform_authentication.dart';

import '../../../core/authentication/presentation/bloc/authentication_bloc.dart';
import '../../../core/utils/constants.dart';

part 'username_authentication_state.dart';

class UserNameAuthenticationCubit extends Cubit<UserNameAuthenticationState> {
  final GetTenantLoginSettingsUsecase _getTenantLoginSettingsUsecase;
  final LoginUsecase _loginUsecase;
  final AuthenticationBloc _authenticationBloc;

  UserNameAuthenticationCubit({
    required getTenantLoginSettingsUsecase,
    required loginUsecase,
    required authenticationBloc,
  })  : _getTenantLoginSettingsUsecase = getTenantLoginSettingsUsecase,
        _loginUsecase = loginUsecase,
        _authenticationBloc = authenticationBloc,
        super(UserNameAuthenticationState.initial());

  void onUserNameChanged(String text) {
    emit(state.copyWith(username: text));
  }

  void onPasswordChanged(String text) {
    emit(state.copyWith(password: text));
  }

  FutureOr<void> login() async {
    emit(state.copyWith(
      status: NetworkStatus.loading,
      errorType: null,
    ));

    try {
      final userLogin = UserLogin(
        username: state.username,
        password: state.password,
      );

      final result = await _loginUsecase(userLogin);

      // MFA flow
      if (result.mfaInfo != null) {
        emit(
          state.copyWith(
            mfaInfo: result.mfaInfo,
            authenticationFlow: AuthenticationFlow.mfa,
          ),
        );

        return emit(
          state.copyWith(
            authenticationFlow: AuthenticationFlow.password,
            mfaInfo: state.mfaInfo,
          ),
        );
      }

      // Reset password flow
      if (result.resetPasswordInfo != null) {
        emit(
          state.copyWith(
            authenticationFlow: AuthenticationFlow.resetPassword,
            authenticationResponse: result,
          ),
        );

        return emit(
          state.copyWith(
            authenticationFlow: AuthenticationFlow.password,
          ),
        );
      }

      // Password flow
      if (result.token != null) {
        _authenticationBloc.add(
          AuthenticationStatusChanged(
            AuthenticationStatus.authenticated,
            authenticationResponse: result,
          ),
        );

        return emit(state.copyWith(errorType: null));
      }

      return emit(
        state.copyWith(
          status: NetworkStatus.idle,
          errorType: ErrorType.unknown,
        ),
      );
    } on UnauthorizedException catch (_) {
      emit(state.copyWith(
        status: NetworkStatus.idle,
        tenantLoginSettings: null,
        errorType: ErrorType.unauthorized,
      ));
    } catch (_) {
      emit(state.copyWith(
        status: NetworkStatus.idle,
        tenantLoginSettings: null,
        errorType: ErrorType.unknown,
      ));
    } finally {
      emit(state.copyWith(
        status: NetworkStatus.idle,
        errorType: null,
        mfaInfo: state.mfaInfo,
      ));
    }
  }

  FutureOr<void> getTenantLoginSettings() async {
    if (state.username.isEmpty) {
      return;
    }

    emit(state.copyWith(
      status: NetworkStatus.loading,
      errorType: null,
      authenticationFlow: AuthenticationFlow.unknown,
    ));

    try {
      final tenantLogin =
          TenantLogin(tenantDomain: state.username.split('@')[1]);
      final result = await _getTenantLoginSettingsUsecase(tenantLogin);

      final authenticationFlow = result.loginProviders.saml?.url != null
          ? AuthenticationFlow.saml
          : AuthenticationFlow.password;

      emit(state.copyWith(
        status: NetworkStatus.idle,
        tenantLoginSettings: result,
        errorType: null,
        authenticationFlow: authenticationFlow,
      ));
    } on TenantNotFoundException catch (_) {
      emit(state.copyWith(
        status: NetworkStatus.idle,
        tenantLoginSettings: null,
        errorType: ErrorType.tenantNotFound,
        authenticationFlow: AuthenticationFlow.unknown,
      ));
    } catch (_) {
      emit(state.copyWith(
        status: NetworkStatus.idle,
        tenantLoginSettings: null,
        errorType: ErrorType.unknown,
        authenticationFlow: AuthenticationFlow.unknown,
      ));
    } finally {
      emit(state.copyWith(
        errorType: null,
      ));
    }
  }
}
