import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mocktail/mocktail.dart';
import 'package:senior_platform_authentication/senior_platform_authentication.dart';
import 'package:senior_platform_authentication_ui/src/core/authentication/data/datasources/secure_storage_local_datasource.dart';
import 'package:senior_platform_authentication_ui/src/core/authentication/domain/entities/secure_storage_keys.dart';

import '../../../../../mocks/token_mock.dart';
import '../../../../../mocks/user_mock.dart';
import '../../../../../utils/string_extension.dart';

class MockFlutterSecureStorage extends Mock implements FlutterSecureStorage {}

void main() {
  late final FlutterSecureStorage storage;
  late final SecureStorageDatasource datasource;

  setUpAll(() {
    storage = MockFlutterSecureStorage();
    datasource = SecureStorageLocalDatasource(storage: storage);
  });

  group('readUser', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => storage.read(
          key: SecureStorageKeys.user,
        ),
      ).thenAnswer((_) async {
        return userModelJson;
      });

      // Act
      final result = await datasource.readUser();

      // Assert
      expect(result, isA<UserModel?>());

      verify(
        () => storage.read(key: SecureStorageKeys.user),
      ).called(1);
    });

    test('storage throws', () async {
      // Arrange
      when(
        () => storage.read(key: SecureStorageKeys.user),
      ).thenThrow(Exception('oops'));

      // Act
      final action = datasource.readUser();

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      verify(
        () => storage.read(key: SecureStorageKeys.user),
      ).called(1);
    });
  });

  group('readToken', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => storage.read(
          key: SecureStorageKeys.token,
        ),
      ).thenAnswer((_) async {
        return tokenModelJson;
      });

      // Act
      final result = await datasource.readToken();

      // Assert
      expect(result, isA<TokenModel?>());

      verify(
        () => storage.read(key: SecureStorageKeys.token),
      ).called(1);
    });

    test('storage throws', () async {
      // Arrange
      when(
        () => storage.read(key: SecureStorageKeys.token),
      ).thenThrow(Exception('oops'));

      // Act
      final action = datasource.readToken();

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      verify(
        () => storage.read(key: SecureStorageKeys.token),
      ).called(1);
    });
  });

  group('readTokenExpirationDate', () {
    DateTime tTokenExpirationDate = DateTime.now();
    test('should execute successfully', () async {
      // Arrange
      when(
        () => storage.read(
          key: SecureStorageKeys.tokenExpirationDate,
        ),
      ).thenAnswer((_) async {
        return tTokenExpirationDate.toIso8601String();
      });

      // Act
      final result = await datasource.readTokenExpirationDate();

      // Assert
      expect(result, isA<DateTime?>());

      verify(
        () => storage.read(key: SecureStorageKeys.tokenExpirationDate),
      ).called(1);
    });

    test('storage throws', () async {
      // Arrange
      when(
        () => storage.read(key: SecureStorageKeys.tokenExpirationDate),
      ).thenThrow(Exception('oops'));

      // Act
      final action = datasource.readTokenExpirationDate();

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      verify(
        () => storage.read(key: SecureStorageKeys.tokenExpirationDate),
      ).called(1);
    });
  });

  group('writeUser', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => storage.write(
          key: any(named: 'key'),
          value: any(named: 'value'),
        ),
      ).thenAnswer((_) async {
        return;
      });

      // Act
      await datasource.writeUser(userModelMock);

      // Assert
      verify(
        () => storage.write(
          key: SecureStorageKeys.user,
          value: userModelJson.clearJson(),
        ),
      ).called(1);
    });

    test('storage throws', () async {
      // Arrange
      when(
        () => storage.write(
          key: any(named: 'key'),
          value: any(named: 'value'),
        ),
      ).thenThrow(Exception('oops'));

      // Act
      final action = datasource.writeUser(userModelMock);

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      verify(
        () => storage.write(
          key: SecureStorageKeys.user,
          value: userModelJson.clearJson(),
        ),
      ).called(1);
    });
  });

  group('writeToken', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => storage.write(
          key: any(named: 'key'),
          value: any(named: 'value'),
        ),
      ).thenAnswer((_) async {
        return;
      });

      // Act
      await datasource.writeToken(tokenModelMock);

      // Assert
      verify(
        () => storage.write(
          key: SecureStorageKeys.token,
          value: tokenModelJson.clearJson(),
        ),
      ).called(1);
    });

    test('storage throws', () async {
      // Arrange
      when(
        () => storage.write(
          key: any(named: 'key'),
          value: any(named: 'value'),
        ),
      ).thenThrow(Exception('oops'));

      // Act
      final action = datasource.writeToken(tokenModelMock);

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      verify(
        () => storage.write(
          key: SecureStorageKeys.token,
          value: tokenModelJson.clearJson(),
        ),
      ).called(1);
    });
  });

  group('writeTokenExpirationDate', () {
    DateTime tTokenExpirationDate = DateTime.now();
    test('should execute successfully', () async {
      // Arrange
      when(
        () => storage.write(
          key: any(named: 'key'),
          value: any(named: 'value'),
        ),
      ).thenAnswer((_) async {
        return;
      });

      // Act
      await datasource.writeTokenExpirationDate(tTokenExpirationDate);

      // Assert
      verify(
        () => storage.write(
          key: SecureStorageKeys.tokenExpirationDate,
          value: tTokenExpirationDate.toIso8601String(),
        ),
      ).called(1);
    });

    test('storage throws', () async {
      // Arrange
      when(
        () => storage.write(
          key: any(named: 'key'),
          value: any(named: 'value'),
        ),
      ).thenThrow(Exception('oops'));

      // Act
      final action = datasource.writeTokenExpirationDate(tTokenExpirationDate);

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      verify(
        () => storage.write(
          key: SecureStorageKeys.tokenExpirationDate,
          value: tTokenExpirationDate.toIso8601String(),
        ),
      ).called(1);
    });
  });

  group('deleteUser', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => storage.delete(key: any(named: 'key')),
      ).thenAnswer((_) async {
        return;
      });

      // Act
      await datasource.deleteUser();

      // Assert
      verify(
        () => storage.delete(key: SecureStorageKeys.user),
      ).called(1);
    });

    test('storage throws', () async {
      // Arrange
      when(
        () => storage.delete(key: any(named: 'key')),
      ).thenThrow(Exception('oops'));

      // Act
      final action = datasource.deleteUser();

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      verify(
        () => storage.delete(key: SecureStorageKeys.user),
      ).called(1);
    });
  });

  group('deleteToken', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => storage.delete(key: any(named: 'key')),
      ).thenAnswer((_) async {
        return;
      });

      // Act
      await datasource.deleteToken();

      // Assert
      verify(
        () => storage.delete(key: SecureStorageKeys.token),
      ).called(1);
    });

    test('storage throws', () async {
      // Arrange
      when(
        () => storage.delete(key: any(named: 'key')),
      ).thenThrow(Exception('oops'));

      // Act
      final action = datasource.deleteToken();

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      verify(
        () => storage.delete(key: SecureStorageKeys.token),
      ).called(1);
    });
  });

  group('deleteTokenExpirationDate', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => storage.delete(key: any(named: 'key')),
      ).thenAnswer((_) async {
        return;
      });

      // Act
      await datasource.deleteTokenExpirationDate();

      // Assert
      verify(
        () => storage.delete(key: SecureStorageKeys.tokenExpirationDate),
      ).called(1);
    });

    test('storage throws', () async {
      // Arrange
      when(
        () => storage.delete(key: any(named: 'key')),
      ).thenThrow(Exception('oops'));

      // Act
      final action = datasource.deleteTokenExpirationDate();

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      verify(
        () => storage.delete(key: SecureStorageKeys.tokenExpirationDate),
      ).called(1);
    });
  });
}
