import 'package:flutter_test/flutter_test.dart';
import 'package:mocktail/mocktail.dart';
import 'package:senior_platform_authentication/senior_platform_authentication.dart';
import 'package:senior_platform_authentication_ui/src/core/authentication/data/datasources/secure_storage_local_datasource.dart';
import 'package:senior_platform_authentication_ui/src/core/authentication/data/repositories/secure_storage_repository.dart';

import '../../../../../mocks/token_mock.dart';
import '../../../../../mocks/user_mock.dart';

class MockSecureStorageDatasource extends Mock
    implements SecureStorageDatasource {}

void main() {
  late final SecureStorageDatasource datasource;
  late final SecureStorageRepository repository;

  setUpAll(() {
    datasource = MockSecureStorageDatasource();
    repository = SecureStorageRepositoryImpl(datasource: datasource);
    registerFallbackValue(userModelMock);
    registerFallbackValue(tokenModelMock);
  });

  group('readUser', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.readUser(),
      ).thenAnswer((_) async {
        return userModelMock;
      });

      verifyNever(
        () => datasource.readUser(),
      );

      // Act
      final result = await repository.readUser();

      // Assert
      verify(
        () => datasource.readUser(),
      ).called(1);

      expect(result, isA<User>());
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.readUser(),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.readUser(),
      );

      // Act
      final action = repository.readUser();

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      // Assert
      verify(
        () => datasource.readUser(),
      ).called(1);
    });
  });

  group('writeUser', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.writeUser(any()),
      ).thenAnswer((_) async {
        return;
      });

      verifyNever(
        () => datasource.writeUser(any()),
      );

      // Act
      await repository.writeUser(userMock);

      // Assert
      verify(
        () => datasource.writeUser(any()),
      ).called(1);
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.writeUser(any()),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.writeUser(any()),
      );

      // Act
      final action = repository.writeUser(userMock);

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      // Assert
      verify(
        () => datasource.writeUser(any()),
      ).called(1);
    });
  });

  group('readToken', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.readToken(),
      ).thenAnswer((_) async {
        return tokenModelMock;
      });

      verifyNever(
        () => datasource.readToken(),
      );

      // Act
      final result = await repository.readToken();

      // Assert
      verify(
        () => datasource.readToken(),
      ).called(1);

      expect(result, isA<Token?>());
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.readToken(),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.readToken(),
      );

      // Act
      final action = repository.readToken();

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      // Assert
      verify(
        () => datasource.readToken(),
      ).called(1);
    });
  });

  group('readTokenExpirationDate', () {
    DateTime tTokenExpirationDate = DateTime.now();
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.readTokenExpirationDate(),
      ).thenAnswer((_) async {
        return tTokenExpirationDate;
      });

      verifyNever(
        () => datasource.readTokenExpirationDate(),
      );

      // Act
      final result = await repository.readTokenExpirationDate();

      // Assert
      verify(
        () => datasource.readTokenExpirationDate(),
      ).called(1);

      expect(result, isA<DateTime?>());
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.readTokenExpirationDate(),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.readTokenExpirationDate(),
      );

      // Act
      final action = repository.readTokenExpirationDate();

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      // Assert
      verify(
        () => datasource.readTokenExpirationDate(),
      ).called(1);
    });
  });

  group('writeToken', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.writeToken(any()),
      ).thenAnswer((_) async {
        return;
      });

      verifyNever(
        () => datasource.writeToken(any()),
      );

      // Act
      await repository.writeToken(tokenMock);

      // Assert
      verify(
        () => datasource.writeToken(any()),
      ).called(1);
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.writeToken(any()),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.writeToken(any()),
      );

      // Act
      final action = repository.writeToken(tokenMock);

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      // Assert
      verify(
        () => datasource.writeToken(any()),
      ).called(1);
    });
  });

  group('writeTokenExpirationDate', () {
    DateTime tTokenExpirationDate = DateTime.now();
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.writeTokenExpirationDate(any()),
      ).thenAnswer((_) async {
        return;
      });

      verifyNever(
        () => datasource.writeTokenExpirationDate(any()),
      );

      // Act
      await repository.writeTokenExpirationDate(tTokenExpirationDate);

      // Assert
      verify(
        () => datasource.writeTokenExpirationDate(any()),
      ).called(1);
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.writeTokenExpirationDate(any()),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.writeTokenExpirationDate(any()),
      );

      // Act
      final action = repository.writeTokenExpirationDate(tTokenExpirationDate);

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      // Assert
      verify(
        () => datasource.writeTokenExpirationDate(any()),
      ).called(1);
    });
  });

  group('deleteUser', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.deleteUser(),
      ).thenAnswer((_) async {
        return;
      });

      verifyNever(
        () => datasource.deleteUser(),
      );

      // Act
      await repository.deleteUser();

      // Assert
      verify(
        () => datasource.deleteUser(),
      ).called(1);
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.deleteUser(),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.deleteUser(),
      );

      // Act
      final action = repository.deleteUser();

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      verify(
        () => datasource.deleteUser(),
      ).called(1);
    });
  });

  group('deleteToken', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.deleteToken(),
      ).thenAnswer((_) async {
        return;
      });

      verifyNever(
        () => datasource.deleteToken(),
      );

      // Act
      await repository.deleteToken();

      // Assert
      verify(
        () => datasource.deleteToken(),
      ).called(1);
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.deleteToken(),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.deleteToken(),
      );

      // Act
      final action = repository.deleteToken();

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      verify(
        () => datasource.deleteToken(),
      ).called(1);
    });
  });

  group('deleteTokenExpirationDate', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => datasource.deleteTokenExpirationDate(),
      ).thenAnswer((_) async {
        return;
      });

      verifyNever(
        () => datasource.deleteTokenExpirationDate(),
      );

      // Act
      await repository.deleteTokenExpirationDate();

      // Assert
      verify(
        () => datasource.deleteTokenExpirationDate(),
      ).called(1);
    });

    test('should throw exception when datasource throws', () async {
      // Arrange
      when(
        () => datasource.deleteTokenExpirationDate(),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => datasource.deleteTokenExpirationDate(),
      );

      // Act
      final action = repository.deleteTokenExpirationDate();

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      verify(
        () => datasource.deleteTokenExpirationDate(),
      ).called(1);
    });
  });
}
