import 'package:flutter_test/flutter_test.dart';
import 'package:mocktail/mocktail.dart';
import 'package:senior_platform_authentication/senior_platform_authentication.dart';
import 'package:senior_platform_authentication_ui/src/core/authentication/data/repositories/secure_storage_repository.dart';
import 'package:senior_platform_authentication_ui/src/core/authentication/domain/usecases/check_stored_token_usecase.dart';

import '../../../../../mocks/token_mock.dart';
import '../../../../../mocks/user_mock.dart';

class MockSecureStorageRepository extends Mock
    implements SecureStorageRepository {}

class MockGetUserUsecase extends Mock implements GetUserUsecase {}

void main() {
  late CheckStoredTokenUsecase checkStoredTokenUsecase;
  late SecureStorageRepository repository;
  late GetUserUsecase getUserUsecase;

  setUp(() {
    repository = MockSecureStorageRepository();
    getUserUsecase = MockGetUserUsecase();
    checkStoredTokenUsecase = CheckStoredTokenUsecase(
        secureStorageRepository: repository, getUserUsecase: getUserUsecase);
  });

  group('checkStoredTokenUsecase', () {
    DateTime tTokenExpirationDate =
        DateTime.now().add(const Duration(minutes: 10));
    test('should execute successfully', () async {
      // Arrange
      when(() => repository.readToken()).thenAnswer((_) async {
        return tokenMock;
      });

      when(() => getUserUsecase.call(any())).thenAnswer((_) async {
        return userMock;
      });

      when(() => repository.readTokenExpirationDate()).thenAnswer((_) async {
        return tTokenExpirationDate;
      });

      verifyNever(() => repository.readToken());

      // Act
      final result = await checkStoredTokenUsecase(NoParams());

      // Assert
      verify(
        () => repository.readToken(),
      ).called(1);

      expect(result, true);
    });

    test('should catch exception when repository throws', () async {
      // Arrange
      when(() => repository.readToken()).thenThrow(Exception('oops'));
      when(() => getUserUsecase.call(any())).thenAnswer((_) async {
        return userMock;
      });

      verifyNever(() => repository.readToken());

      // Act
      final result = await checkStoredTokenUsecase(NoParams());

      // Assert
      expect(result, false);

      // Assert
      verify(
        () => repository.readToken(),
      ).called(1);
    });

    test('should verify if token is valid', () async {
      // Arrange
      when(() => repository.readToken()).thenAnswer((_) async {
        return null;
      });

      when(() => getUserUsecase.call(any())).thenAnswer((_) async {
        return userMock;
      });

      verifyNever(
        () => repository.readToken(),
      );

      // Act
      final result = await checkStoredTokenUsecase(NoParams());

      // Assert
      verify(
        () => repository.readToken(),
      ).called(1);

      expect(result, false);
    });

    test('should verify if accessToken is valid', () async {
      // Arrange
      when(() => repository.readToken()).thenAnswer((_) async {
        return tokenMock.copyWith(accessToken: '');
      });

      when(() => getUserUsecase.call(any())).thenAnswer((_) async {
        return userMock;
      });

      verifyNever(() => repository.readToken());

      // Act
      final result = await checkStoredTokenUsecase(NoParams());

      // Assert
      verify(
        () => repository.readToken(),
      ).called(1);

      expect(result, false);
    });

    test('should verify if refreshToken is valid', () async {
      // Arrange
      when(() => repository.readToken()).thenAnswer((_) async {
        return tokenMock.copyWith(refreshToken: '');
      });

      when(() => getUserUsecase.call(any())).thenAnswer((_) async {
        return userMock;
      });

      verifyNever(() => repository.readToken());

      // Act
      final result = await checkStoredTokenUsecase(NoParams());

      // Assert
      verify(
        () => repository.readToken(),
      ).called(1);

      expect(result, false);
    });

    test('should verify if tokenExpirationDate is valid', () async {
      // Arrange
      when(() => repository.readTokenExpirationDate()).thenAnswer((_) async {
        return DateTime.fromMillisecondsSinceEpoch(1);
      });

      when(() => getUserUsecase.call(any())).thenAnswer((_) async {
        return userMock;
      });

      verifyNever(() => repository.readTokenExpirationDate());

      // Act
      final result = await checkStoredTokenUsecase(NoParams());

      // Assert
      verify(
        () => repository.readToken(),
      ).called(1);

      expect(result, false);
    });

    test('should return false when token is not valid', () async {
      // Arrange
      when(() => repository.readTokenExpirationDate()).thenAnswer((_) async {
        return DateTime.fromMillisecondsSinceEpoch(1);
      });

      when(() => getUserUsecase.call(any())).thenThrow(UnauthorizedException());

      verifyNever(() => repository.readTokenExpirationDate());

      // Act
      final result = await checkStoredTokenUsecase(NoParams());

      // Assert
      verify(
        () => repository.readToken(),
      ).called(1);

      expect(result, false);
    });
  });
}
