import 'package:bloc_test/bloc_test.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mocktail/mocktail.dart';
import 'package:senior_platform_authentication_ui/senior_platform_authentication_ui.dart';

import '../../../../../mocks/authentication_data_mock.dart';
import '../../../../../mocks/token_mock.dart';
import '../../../../../mocks/user_mock.dart';

class MockGetUserUsecase extends Mock implements GetUserUsecase {}

class MockStoreAuthenticationDataUsecase extends Mock
    implements StoreAuthenticationDataUsecase {}

class MockCheckStoredTokenUsecase extends Mock
    implements CheckStoredTokenUsecase {}

class MockLogoutUsecase extends Mock implements LogoutUsecase {}

class MockClearStoredDataUsecase extends Mock
    implements ClearStoredDataUsecase {}

class MockGetStoredTokenUsecase extends Mock implements GetStoredTokenUsecase {}

class MockRefreshStoredTokenUsecase extends Mock
    implements RefreshStoredTokenUsecase {}

void main() {
  late GetUserUsecase getUserUsecase;
  late StoreAuthenticationDataUsecase storeAuthenticationDataUsecase;
  late CheckStoredTokenUsecase checkStoredTokenUsecase;
  late LogoutUsecase logoutUsecase;
  late ClearStoredDataUsecase clearStoredDataUsecase;
  late GetStoredTokenUsecase getStoredTokenUsecase;
  late AuthenticationBloc authenticationBloc;
  late RefreshStoredTokenUsecase refreshStoredTokenUsecase;

  setUp(() {
    getUserUsecase = MockGetUserUsecase();
    storeAuthenticationDataUsecase = MockStoreAuthenticationDataUsecase();
    checkStoredTokenUsecase = MockCheckStoredTokenUsecase();
    logoutUsecase = MockLogoutUsecase();
    getStoredTokenUsecase = MockGetStoredTokenUsecase();
    clearStoredDataUsecase = MockClearStoredDataUsecase();
    refreshStoredTokenUsecase = MockRefreshStoredTokenUsecase();
    authenticationBloc = AuthenticationBloc.mocked(
      getUserUsecase: getUserUsecase,
      storeAuthenticationDataUsecase: storeAuthenticationDataUsecase,
      checkStoredTokenUsecase: checkStoredTokenUsecase,
      logoutUsecase: logoutUsecase,
      clearStoredDataUsecase: clearStoredDataUsecase,
      getStoredTokenUsecase: getStoredTokenUsecase,
      refreshStoredTokenUsecase: refreshStoredTokenUsecase,
    );
    registerFallbackValue(authenticationDataMock);
    registerFallbackValue(NoParams());
  });

  group('factory', () {
    test('AuthenticationBloc factory', () {
      final leafAuthenticationBloc = AuthenticationBloc();
      expect(leafAuthenticationBloc, isA<AuthenticationBloc>());
    });
  });

  group('onAuthenticationStatusChanged', () {
    blocTest<AuthenticationBloc, AuthenticationState>(
      'emits [AuthenticationState.unknown] when AuthenticationStatusChanged '
      'event is fired with AuthenticationStatus.unknown',
      build: () => authenticationBloc,
      act: (bloc) => bloc
          .add(const AuthenticationStatusChanged(AuthenticationStatus.unknown)),
      expect: () => <AuthenticationState>[
        const AuthenticationState.unknown(),
      ],
      verify: (_) async {
        verifyNever(() => getUserUsecase.call(''));
      },
    );

    blocTest<AuthenticationBloc, AuthenticationState>(
      'emits [AuthenticationState.unauthenticated] when AuthenticationStatusChanged '
      'event is fired with AuthenticationStatus.unauthenticated',
      build: () => authenticationBloc,
      act: (bloc) => bloc.add(const AuthenticationStatusChanged(
          AuthenticationStatus.unauthenticated)),
      expect: () => <AuthenticationState>[
        const AuthenticationState.unauthenticated(),
      ],
      verify: (_) async {
        verifyNever(() => getUserUsecase.call(''));
      },
    );

    blocTest<AuthenticationBloc, AuthenticationState>(
      'emits [AuthenticationState.authenticated] when AuthenticationStatusChanged '
      'event is fired with AuthenticationStatus.authenticated '
      'and verify usecases call',
      setUp: () {
        when(() => getUserUsecase.call(any()))
            .thenAnswer((_) async => userMock);
        when(() => storeAuthenticationDataUsecase.call(any()))
            .thenAnswer((_) async => true);
      },
      build: () => authenticationBloc,
      act: (bloc) => bloc.add(const AuthenticationStatusChanged(
        AuthenticationStatus.authenticated,
        authenticationResponse: authenticationResponseMock,
      )),
      expect: () => <AuthenticationState>[
        const AuthenticationState.authenticated(),
      ],
      verify: (_) async {
        verify(() => getUserUsecase.call(any())).called(1);
        verify(() => storeAuthenticationDataUsecase.call(any())).called(1);
      },
    );
  });

  group('onLogoutRequested', () {
    blocTest<AuthenticationBloc, AuthenticationState>(
      'emits [AuthenticationState.unauthenticated] when LogoutRequested '
      'event if fired and verify usecases call',
      setUp: () {
        when(() => getStoredTokenUsecase.call(any()))
            .thenAnswer((_) async => tokenMock);
        when(() => logoutUsecase.call(any())).thenAnswer((_) async => true);
        when(() => clearStoredDataUsecase.call(any()))
            .thenAnswer((_) async => true);
      },
      build: () => authenticationBloc,
      act: (bloc) => bloc.add(LogoutRequested()),
      expect: () => <AuthenticationState>[
        const AuthenticationState.unauthenticated(),
      ],
      verify: (_) async {
        verify(() => getStoredTokenUsecase.call(any())).called(1);
        verify(() => logoutUsecase.call(any())).called(1);
        verify(() => clearStoredDataUsecase.call(any())).called(1);
      },
    );
  });

  group('onCheckAuthenticationRequested', () {
    blocTest<AuthenticationBloc, AuthenticationState>(
      'emits [AuthenticationState.authenticated] when CheckAuthenticationRequested '
      'event if fired with valid token on storage and verify usecases call',
      setUp: () {
        when(() => checkStoredTokenUsecase.call(any()))
            .thenAnswer((_) async => true);
      },
      build: () => authenticationBloc,
      act: (bloc) => bloc.add(CheckAuthenticationRequested()),
      expect: () => <AuthenticationState>[
        const AuthenticationState.authenticated(),
      ],
      verify: (_) async {
        verify(() => checkStoredTokenUsecase.call(any())).called(1);
      },
    );

    blocTest<AuthenticationBloc, AuthenticationState>(
      'emits [AuthenticationState.unauthenticated] when CheckAuthenticationRequested '
      'event if fired with invalid token on storage and verify usecases call',
      setUp: () {
        when(() => checkStoredTokenUsecase.call(any()))
            .thenAnswer((_) async => false);
        when(() => refreshStoredTokenUsecase.call(any()))
            .thenAnswer((_) async => null);
      },
      build: () => authenticationBloc,
      act: (bloc) => bloc.add(CheckAuthenticationRequested()),
      expect: () => <AuthenticationState>[
        const AuthenticationState.unauthenticated(),
      ],
      verify: (_) async {
        verify(() => checkStoredTokenUsecase.call(any())).called(1);
      },
    );
  });
}
