import 'package:bloc_test/bloc_test.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mocktail/mocktail.dart';
import 'package:senior_platform_authentication_ui/senior_platform_authentication_ui.dart';

import '../../../../mocks/mfa_info_mock.dart';
import '../../../../mocks/tenant_login_settings_mock.dart';
import '../../../../mocks/token_mock.dart';

class MockGetTenantLoginSettingsUsecase extends Mock
    implements GetTenantLoginSettingsUsecase {}

class MockLoginUsecase extends Mock implements LoginUsecase {}

class MockAuthenticationBloc extends Mock implements AuthenticationBloc {}

void main() {
  late GetTenantLoginSettingsUsecase getTenantLoginSettingsUsecase;
  late LoginUsecase loginUsecase;
  late AuthenticationBloc authenticationBloc;
  late UserNameAuthenticationCubit userNameAuthenticationCubit;
  const tUsername = 'teste@senior.com.br';
  const tPassword = 'Teste1!';

  setUp(() {
    getTenantLoginSettingsUsecase = MockGetTenantLoginSettingsUsecase();
    loginUsecase = MockLoginUsecase();
    authenticationBloc = MockAuthenticationBloc();
    userNameAuthenticationCubit = UserNameAuthenticationCubit(
      authenticationBloc: authenticationBloc,
      loginUsecase: loginUsecase,
      getTenantLoginSettingsUsecase: getTenantLoginSettingsUsecase,
    );
  });

  group('onUserNameChanged', () {
    blocTest<UserNameAuthenticationCubit, UserNameAuthenticationState>(
      'emits current state with new userName when onUserNameChanged '
      'event is fired with userName text',
      build: () => userNameAuthenticationCubit,
      seed: () => UserNameAuthenticationState.initial(),
      act: (cubit) => cubit.onUserNameChanged(tUsername),
      expect: () => <UserNameAuthenticationState>[
        UserNameAuthenticationState.initial().copyWith(username: tUsername),
      ],
    );
  });

  group('onPasswordChanged', () {
    blocTest<UserNameAuthenticationCubit, UserNameAuthenticationState>(
      'emits current state with new userName when onUserNameChanged '
      'event is fired with userName text',
      build: () => userNameAuthenticationCubit,
      seed: () => UserNameAuthenticationState.initial(),
      act: (cubit) => cubit.onPasswordChanged(tPassword),
      expect: () => <UserNameAuthenticationState>[
        UserNameAuthenticationState.initial().copyWith(password: tPassword),
      ],
    );
  });

  group('login', () {
    const tUserLogin = UserLogin(
      username: tUsername,
      password: tPassword,
    );
    blocTest<UserNameAuthenticationCubit, UserNameAuthenticationState>(
      'emits correct states when login executes successfully',
      setUp: () => when(() => loginUsecase.call(tUserLogin))
          .thenAnswer((_) async => authenticationResponseMock),
      build: () => userNameAuthenticationCubit,
      seed: () => UserNameAuthenticationState(
        status: NetworkStatus.idle,
        username: tUsername,
        password: tPassword,
        tenantLoginSettings: tenantLoginSettingsMock,
        errorType: null,
        authenticationFlow: AuthenticationFlow.password,
      ),
      act: (cubit) => cubit.login(),
      expect: () => <UserNameAuthenticationState>[
        UserNameAuthenticationState(
          status: NetworkStatus.loading,
          username: tUsername,
          password: tPassword,
          tenantLoginSettings: tenantLoginSettingsMock,
          errorType: null,
          authenticationFlow: AuthenticationFlow.password,
        ),
        UserNameAuthenticationState(
          status: NetworkStatus.idle,
          username: tUsername,
          password: tPassword,
          tenantLoginSettings: tenantLoginSettingsMock,
          errorType: null,
          authenticationFlow: AuthenticationFlow.password,
        ),
      ],
    );

    blocTest<UserNameAuthenticationCubit, UserNameAuthenticationState>(
      'emits correct states when login executes successfully with MFAInfo CONFIGURED',
      setUp: () => when(() => loginUsecase.call(tUserLogin))
          .thenAnswer((_) async => authenticationResponseMfaConfiguredMock),
      build: () => userNameAuthenticationCubit,
      seed: () => UserNameAuthenticationState(
        status: NetworkStatus.idle,
        username: tUsername,
        password: tPassword,
        tenantLoginSettings: tenantLoginSettingsMock,
        errorType: null,
        authenticationFlow: AuthenticationFlow.password,
      ),
      act: (cubit) => cubit.login(),
      expect: () => <UserNameAuthenticationState>[
        UserNameAuthenticationState(
          status: NetworkStatus.loading,
          username: tUsername,
          password: tPassword,
          tenantLoginSettings: tenantLoginSettingsMock,
          errorType: null,
          authenticationFlow: AuthenticationFlow.password,
        ),
        UserNameAuthenticationState(
          status: NetworkStatus.loading,
          username: tUsername,
          password: tPassword,
          tenantLoginSettings: tenantLoginSettingsMock,
          errorType: null,
          authenticationFlow: AuthenticationFlow.mfa,
          mfaInfo: mfaInfoConfiguredMock,
        ),
        UserNameAuthenticationState(
          status: NetworkStatus.loading,
          username: tUsername,
          password: tPassword,
          tenantLoginSettings: tenantLoginSettingsMock,
          errorType: null,
          authenticationFlow: AuthenticationFlow.password,
          mfaInfo: mfaInfoConfiguredMock,
        ),
        UserNameAuthenticationState(
          status: NetworkStatus.idle,
          username: tUsername,
          password: tPassword,
          tenantLoginSettings: tenantLoginSettingsMock,
          errorType: null,
          authenticationFlow: AuthenticationFlow.password,
          mfaInfo: mfaInfoConfiguredMock,
        ),
      ],
    );

    blocTest<UserNameAuthenticationCubit, UserNameAuthenticationState>(
      'emits correct states when login executes successfully with MFAInfo not CONFIGURED',
      setUp: () => when(() => loginUsecase.call(tUserLogin))
          .thenAnswer((_) async => authenticationResponseMfaNotConfiguredMock),
      build: () => userNameAuthenticationCubit,
      seed: () => UserNameAuthenticationState(
        status: NetworkStatus.idle,
        username: tUsername,
        password: tPassword,
        tenantLoginSettings: tenantLoginSettingsMock,
        errorType: null,
        authenticationFlow: AuthenticationFlow.password,
      ),
      act: (cubit) => cubit.login(),
      expect: () => <UserNameAuthenticationState>[
        UserNameAuthenticationState(
          status: NetworkStatus.loading,
          username: tUsername,
          password: tPassword,
          tenantLoginSettings: tenantLoginSettingsMock,
          errorType: null,
          authenticationFlow: AuthenticationFlow.password,
        ),
        UserNameAuthenticationState(
          status: NetworkStatus.loading,
          username: tUsername,
          password: tPassword,
          tenantLoginSettings: tenantLoginSettingsMock,
          errorType: null,
          authenticationFlow: AuthenticationFlow.mfa,
          mfaInfo: mfaInfoNotConfiguredMock,
        ),
        UserNameAuthenticationState(
          status: NetworkStatus.loading,
          username: tUsername,
          password: tPassword,
          tenantLoginSettings: tenantLoginSettingsMock,
          errorType: null,
          authenticationFlow: AuthenticationFlow.password,
          mfaInfo: mfaInfoNotConfiguredMock,
        ),
        UserNameAuthenticationState(
          status: NetworkStatus.idle,
          username: tUsername,
          password: tPassword,
          tenantLoginSettings: tenantLoginSettingsMock,
          errorType: null,
          authenticationFlow: AuthenticationFlow.password,
          mfaInfo: mfaInfoNotConfiguredMock,
        ),
      ],
    );

    blocTest<UserNameAuthenticationCubit, UserNameAuthenticationState>(
      'emits correct states when login fails with UnauthorizedException',
      setUp: () => when(() => loginUsecase.call(tUserLogin))
          .thenThrow(UnauthorizedException()),
      build: () => userNameAuthenticationCubit,
      seed: () => UserNameAuthenticationState(
        status: NetworkStatus.idle,
        username: tUsername,
        password: tPassword,
        tenantLoginSettings: tenantLoginSettingsMock,
        errorType: null,
        authenticationFlow: AuthenticationFlow.unknown,
      ),
      act: (cubit) => cubit.login(),
      expect: () => <UserNameAuthenticationState>[
        UserNameAuthenticationState(
          status: NetworkStatus.loading,
          username: tUsername,
          password: tPassword,
          tenantLoginSettings: tenantLoginSettingsMock,
          errorType: null,
          authenticationFlow: AuthenticationFlow.unknown,
        ),
        UserNameAuthenticationState(
          status: NetworkStatus.idle,
          username: tUsername,
          password: tPassword,
          tenantLoginSettings: tenantLoginSettingsMock,
          errorType: ErrorType.unauthorized,
          authenticationFlow: AuthenticationFlow.unknown,
        ),
        UserNameAuthenticationState(
          status: NetworkStatus.idle,
          username: tUsername,
          password: tPassword,
          tenantLoginSettings: tenantLoginSettingsMock,
          errorType: null,
          authenticationFlow: AuthenticationFlow.unknown,
        ),
      ],
    );

    blocTest<UserNameAuthenticationCubit, UserNameAuthenticationState>(
      'emits correct states when login fails with Unknown error',
      setUp: () => when(() => loginUsecase.call(tUserLogin))
          .thenThrow(Exception('Opps')),
      build: () => userNameAuthenticationCubit,
      seed: () => UserNameAuthenticationState(
        status: NetworkStatus.idle,
        username: tUsername,
        password: tPassword,
        tenantLoginSettings: tenantLoginSettingsMock,
        errorType: null,
        authenticationFlow: AuthenticationFlow.unknown,
      ),
      act: (cubit) => cubit.login(),
      expect: () => <UserNameAuthenticationState>[
        UserNameAuthenticationState(
          status: NetworkStatus.loading,
          username: tUsername,
          password: tPassword,
          tenantLoginSettings: tenantLoginSettingsMock,
          errorType: null,
          authenticationFlow: AuthenticationFlow.unknown,
        ),
        UserNameAuthenticationState(
          status: NetworkStatus.idle,
          username: tUsername,
          password: tPassword,
          tenantLoginSettings: tenantLoginSettingsMock,
          errorType: ErrorType.unknown,
          authenticationFlow: AuthenticationFlow.unknown,
        ),
        UserNameAuthenticationState(
          status: NetworkStatus.idle,
          username: tUsername,
          password: tPassword,
          tenantLoginSettings: tenantLoginSettingsMock,
          errorType: null,
          authenticationFlow: AuthenticationFlow.unknown,
        ),
      ],
    );
  });

  group('getTenantLoginSettings', () {
    TenantLogin tTenantLogin =
        TenantLogin(tenantDomain: tUsername.split('@')[1]);
    blocTest<UserNameAuthenticationCubit, UserNameAuthenticationState>(
      'emits correct states when getTenantLoginSettings executes successfully',
      setUp: () => when(() => getTenantLoginSettingsUsecase.call(tTenantLogin))
          .thenAnswer((_) async => tenantLoginSettingsMock),
      build: () => userNameAuthenticationCubit,
      seed: () => const UserNameAuthenticationState(
        status: NetworkStatus.idle,
        username: tUsername,
        password: '',
        tenantLoginSettings: null,
        errorType: null,
        authenticationFlow: AuthenticationFlow.password,
      ),
      act: (cubit) => cubit.getTenantLoginSettings(),
      expect: () => <UserNameAuthenticationState>[
        const UserNameAuthenticationState(
          status: NetworkStatus.loading,
          username: tUsername,
          password: '',
          tenantLoginSettings: null,
          errorType: null,
          authenticationFlow: AuthenticationFlow.unknown,
        ),
        UserNameAuthenticationState(
          status: NetworkStatus.idle,
          username: tUsername,
          password: '',
          tenantLoginSettings: tenantLoginSettingsMock,
          errorType: null,
          authenticationFlow: AuthenticationFlow.password,
        ),
      ],
    );

    blocTest<UserNameAuthenticationCubit, UserNameAuthenticationState>(
      'emits correct states when getTenantLoginSettings fails with TenantNotFoundException',
      setUp: () => when(() => getTenantLoginSettingsUsecase.call(tTenantLogin))
          .thenThrow(TenantNotFoundException()),
      build: () => userNameAuthenticationCubit,
      seed: () => const UserNameAuthenticationState(
        status: NetworkStatus.idle,
        username: tUsername,
        password: '',
        tenantLoginSettings: null,
        errorType: null,
        authenticationFlow: AuthenticationFlow.unknown,
      ),
      act: (cubit) => cubit.getTenantLoginSettings(),
      expect: () => <UserNameAuthenticationState>[
        const UserNameAuthenticationState(
          status: NetworkStatus.loading,
          username: tUsername,
          password: '',
          tenantLoginSettings: null,
          errorType: null,
          authenticationFlow: AuthenticationFlow.unknown,
        ),
        const UserNameAuthenticationState(
          status: NetworkStatus.idle,
          username: tUsername,
          password: '',
          tenantLoginSettings: null,
          errorType: ErrorType.tenantNotFound,
          authenticationFlow: AuthenticationFlow.unknown,
        ),
        const UserNameAuthenticationState(
          status: NetworkStatus.idle,
          username: tUsername,
          password: '',
          tenantLoginSettings: null,
          errorType: null,
          authenticationFlow: AuthenticationFlow.unknown,
        ),
      ],
    );

    blocTest<UserNameAuthenticationCubit, UserNameAuthenticationState>(
      'emits correct states when getTenantLoginSettings fails with Unknown error',
      setUp: () => when(() => getTenantLoginSettingsUsecase.call(tTenantLogin))
          .thenThrow(Exception('Opps')),
      build: () => userNameAuthenticationCubit,
      seed: () => const UserNameAuthenticationState(
        status: NetworkStatus.idle,
        username: tUsername,
        password: '',
        tenantLoginSettings: null,
        errorType: null,
        authenticationFlow: AuthenticationFlow.unknown,
      ),
      act: (cubit) => cubit.getTenantLoginSettings(),
      expect: () => <UserNameAuthenticationState>[
        const UserNameAuthenticationState(
          status: NetworkStatus.loading,
          username: tUsername,
          password: '',
          tenantLoginSettings: null,
          errorType: null,
          authenticationFlow: AuthenticationFlow.unknown,
        ),
        const UserNameAuthenticationState(
          status: NetworkStatus.idle,
          username: tUsername,
          password: '',
          tenantLoginSettings: null,
          errorType: ErrorType.unknown,
          authenticationFlow: AuthenticationFlow.unknown,
        ),
        const UserNameAuthenticationState(
          status: NetworkStatus.idle,
          username: tUsername,
          password: '',
          tenantLoginSettings: null,
          errorType: null,
          authenticationFlow: AuthenticationFlow.unknown,
        ),
      ],
    );
  });
}
